#include "object_api_firmware_info.h"

PyTypeObject api_firmware_info_object_type = {
    PyVarObject_HEAD_INIT(NULL, 0)
    MODULE_NAME "." API_FIRMWARE_INFO_OBJECT_NAME, /* tp_name */
    sizeof(api_firmware_info_object), /* tp_basicsize */
    0,                         /* tp_itemsize */
    (destructor)0,             /* tp_dealloc */
    0,                         /* tp_print */
    (getattrfunc)PyObject_GenericGetAttr, /* tp_getattr */
    (setattrfunc)PyObject_GenericSetAttr, /* tp_setattr */
    0,                         /* tp_reserved */
    0,                         /* tp_repr */
    0,                         /* tp_as_number */
    0,                         /* tp_as_sequence */
    0,                         /* tp_as_mapping */ 
    0,                         /* tp_hash  */
    0,                         /* tp_call */
    0,                         /* tp_str */
    PyObject_GenericGetAttr,   /* tp_getattro */
    PyObject_GenericSetAttr,   /* tp_setattro */
    0,                         /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT |
    Py_TPFLAGS_BASETYPE,       /* tp_flags */
    API_FIRMWARE_INFO_OBJECT_NAME" object",      /* tp_doc */
    0,                         /* tp_traverse */
    0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    0,                         /* tp_methods */
    api_firmware_info_object_members, /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    (initproc)api_firmware_info_object_init, /* tp_init */
    0,                         /* tp_alloc */
    0,                         /* tp_new */
};


bool setup_api_firmware_info_object(PyObject* module)
{
    api_firmware_info_object_type.tp_new = PyType_GenericNew;
    if (PyType_Ready(&api_firmware_info_object_type) < 0) {
        return false;
    }
    Py_INCREF(&api_firmware_info_object_type);
    PyModule_AddObject(module, API_FIRMWARE_INFO_OBJECT_NAME, (PyObject*)&api_firmware_info_object_type);
    return true;
}
